/*****************************************************************************
 *
 * MODULE NAME:    sys_power.c
 * PROJECT CODE:   BlueStream
 * DESCRIPTION:    Low Power Clock Activation
 * MAINTAINER:     Tom Kerwick
 * DATE:           10 December 2001
 *
 * SOURCE CONTROL: $Id: sys_power.c,v 1.12 2011/04/01 16:03:27 garyf Exp $
 *
 * LICENSE:
 *     This source code is copyright (c) 2000-2008 Ceva Inc.
 *     All rights reserved.
 *
 *****************************************************************************/


#include "sys_config.h"
#include "sys_types.h"
#include "sys_const.h"

#include "hw_delay.h"
#include "uslc_chan_ctrl.h"
#include "lslc_irq.h"
#include "sys_hal_features.h"

#include "sys_features.h"
#include "sys_power.h"
#include "lmp_timer.h"
#include "lmp_config.h"
#include "lmp_acl_container.h"
#include "bk_reg.h"
#include "include.h"
#include "DeviceBtj_Header.h"
#include "driver_icu.h"// for warning messge change
#include "app_dmp_module.h"

extern t_queue BTQueues[];
extern u_int32  Check_Inquiry_Is_Busy(void);
extern u_int32 Get_exist_Acl_Num(void);
#if (TRA_HCIT_GENERIC_SUPPORTED==1)
extern void (*hci_generic_exit_hc_callback)(void);
#endif


static u_int8 syspwr_low_power_mode_active;
static u_int32 syspwr_num_frames_to_sleep;

/* can be increased if CPU is slown down and execution takes time */
#define SYS_PWR_MIN_FRAMES_TO_SLEEP      1
#define SYS_PWR_AUX_TIMER_MIN_RESOLUTION 1


u_int8 bt_time_tick;


/*
 * FUNCTION:   SYSpwr_Is_Available_Sleep_System()
 * PURPOSE:    Returns TRUE if system is available to sleep.
 */
int SYSpwr_Is_Available_Sleep_System(void)
{

  /* No need to monitor LMP_OUT, L2CAP_OUT and SCO_OUT queue
   * because once placed data there, dequeued in LC ISR
   * so that halt scheduler at this point is no problem
   * even if containing any data
   */
  t_queue *q = BTQueues;

  do  ///Check for HCI_COMMAND_Q, HCI_EVENT_Q and LMP_IN_Q
  {
    if(q->num_entries)
    {
      return(0/*Not Empty*/);
    }

    q++;
  }
  while(q != &BTQueues[LMP_OUT_Q]);

  q = BTQueues + L2CAP_IN_Q;

  do ///  Check for L2CAP_IN_Q
  {
    if(q->num_entries)
    {
      return(0/*Not Empty*/);
    }

    q++;
  }
  while(q != &BTQueues[L2CAP_OUT_Q]);

  if(Check_Inquiry_Is_Busy())
  {

    return 0;
  }

  return(!LMtmr_Timer_Expired()); //Allow sleep if timers not expired

}


/*****************************************************************************
 * FUNCTION:   SYSpwr_Halt_System
 * PURPOSE:    Turn off entire chip / Deep Sleep
 ****************************************************************************/
extern void app_gpio_sleep(void);
extern void app_gpio_wakeup_set(void);
extern void app_mouse_process(void);
void  SYSpwr_Halt_System(void)
{
  // may add cpu halt/sleep functionality here
  //if((sys_flag&FLAG_CONNECTED) || (sys_flag&FLAG_BLE_CONNECTED))
  {
    if (LC_Is_Sleep_Pending())
    {
      if ( LC_Is_Tabasco_Asleep())
      {
        if(SYSpwr_Is_Available_Sleep_System())
        {
          app_mouse_process();

          if((sys_flag & FLAG_CONNECTED) || (sys_flag & FLAG_BLE_CONNECTED))
          {
            if(syspwr_num_frames_to_sleep > SLEEP_CLOSE_XTAL_TIME)
            {
              //app_gpio_sleep();
              syspwr_low_power_mode_active = 1;

              cpu_close_xtal_sleep();
            }
            else
            {

              cpu_idle_sleep();
            }
          }
          else
          {

            cpu_idle_sleep();
          }

          gpio_wakeup_set();

        }

      }
    }

  }
}


/*****************************************************************************
 * FUNCTION:   SYSpwr_Exit_Halt_System()
 * PURPOSE:    Wake entire chip back up (re-enable any gated clocks etc.)
 ****************************************************************************/
void SYSpwr_Exit_Halt_System(void)
{
  syspwr_low_power_mode_active = 0;
}

/*****************************************************************************
 * FUNCTION:   SYSpwr_Is_Available_Halt_System()
 * PURPOSE:    Returns TRUE if system is available to halt.
 ****************************************************************************/
u_int8 SYSpwr_Is_Available_Halt_System(void)
{
  return syspwr_low_power_mode_active;
}

/*****************************************************************************
 * FUNCTION:   SYSpwr_LowFrequencyOscillatorAvailable()
 * PURPOSE:    Returns TRUE if low freq oscillator is available.
 ****************************************************************************/

t_sys_lf_osc_type SYSpwr_LowFrequencyOscillatorAvailable(void)
{
  //return SYS_LF_OSC_NONE;//SYS_LF_OSC_32K000;//
  return SYS_LF_OSC_32K000;
}

/*****************************************************************************
 * FUNCTION:   SYSpwr_Setup_Sleep_Timer()
 * PURPOSE:    Generic Function to setup sleep timer - can be implemented
 *             using any clock source capable of generating interrupt
 *
 *             this version uses AUX_TIMER
 ****************************************************************************/
extern char updata_time_flag;
extern u_int8 get_sniff_interval(void);
//#include "le_connection.h"

void SYSpwr_Setup_Sleep_Timer(u_int32 max_num_frames_to_sleep)
{
  DEBUG_MSG(0X3F);
  DEBUG_MSG(max_num_frames_to_sleep);
  HW_set_aux_tim_intr_clr(1);

  if (max_num_frames_to_sleep == 1)
  {
    HW_set_aux_timer(1);

    max_num_frames_to_sleep = 0;
  }
  else if (max_num_frames_to_sleep > NUM_FRAMES_TO_SLEEP)
  {
    HW_set_aux_timer(NUM_FRAMES_TO_SLEEP >> AUX_T_SHIFT);
    max_num_frames_to_sleep -= NUM_FRAMES_TO_SLEEP + SYS_PWR_MIN_FRAMES_TO_SLEEP;
  }
  else
  {
    if (max_num_frames_to_sleep > (SYS_PWR_MIN_FRAMES_TO_SLEEP + SYS_PWR_AUX_TIMER_MIN_RESOLUTION))
    {

      // REG_APB5_GPIO3_DATA=REG_APB5_GPIO3_DATA^0x04;

      if(max_num_frames_to_sleep > SLEEP_CLOSE_XTAL_TIME)
      {
        HW_set_aux_timer((max_num_frames_to_sleep - SYS_PWR_MIN_FRAMES_TO_SLEEP * 3) >> AUX_T_SHIFT);
      }
      else
      {
        if((get_sniff_interval() >= 0x0e) && (get_sniff_interval() <= 0x12) )
        {
          HW_set_aux_timer(1);
        }
        else
        {
          HW_set_aux_timer((max_num_frames_to_sleep - SYS_PWR_MIN_FRAMES_TO_SLEEP) >> AUX_T_SHIFT);
        }
      }
    }
    else
    {
      HW_set_aux_timer(SYS_PWR_AUX_TIMER_MIN_RESOLUTION >> AUX_T_SHIFT);
    }

    if((((REG_APB5_GPIO1_DATA >> 8) & 0x02) == 0) || (sys_flag & FLAG_BT_INPAIR)) //pair
    {
      max_num_frames_to_sleep = 0;
    }
  }

  //HW_set_aux_tim_intr_mask(0);
  syspwr_num_frames_to_sleep = max_num_frames_to_sleep;
}

/*****************************************************************************
 * FUNCTION:   SYSpwr_Initialise
 * PURPOSE:
 ****************************************************************************/
void SYSpwr_Initialise(void)
{
  syspwr_num_frames_to_sleep = 0;
  syspwr_low_power_mode_active = 0;
  //Leave low power oscillator disabled until enabled by TCI.

}

/*****************************************************************************
 * FUNCTION:   SYSpwr_Handle_Early_Wakeup
 * PURPOSE:
 ****************************************************************************/
void SYSpwr_Handle_Early_Wakeup(void)
{
  /* wake chip back up to check on UART status */
  //SYSpwr_Exit_Halt_System();
  /*
   * DEPLOYMENT TEAM need to put code into this if() to check for HCI transport
   * activity. If TRUE, then subsequent 'naps' for this sleep period will be aborted
   */
  syspwr_low_power_mode_active = 0;
  HW_set_aux_tim_intr_mask(1);
  LSLCirq_Disable_Aux_Tim_Intr();
  USLCchac_Wakeup(); /* move onto next state, wake system up... */

}

/*****************************************************************************
 * FUNCTION:   SYSpwr_Is_Low_Power_Mode_Active
 * PURPOSE:
 ****************************************************************************/
u_int8 SYSpwr_Is_Low_Power_Mode_Active(void)
{
  return syspwr_low_power_mode_active;
}

/*****************************************************************************
 * FUNCTION:   SYSpwr_Set_LowFrequencyOscillatorAvailable_Value()
 * PURPOSE:    set if oscillator is available.
 *             Returns TRUE if request was processed.
 ****************************************************************************/
boolean SYSpwr_Set_LowFrequencyOscillatorAvailable_Value(u_int8 osc_available)
{
  if((SYS_LF_OSCILLATOR_PRESENT == 0) && (osc_available == 1))
  {
    return FALSE;
  }
  else

  {
    return TRUE;
  }
}

/*****************************************************************************
 * SYSpwr_Get_Min_Frames_To_Sleep
 * minimum amount of frames that device can be put in deep sleep
 ****************************************************************************/
u_int32 SYSpwr_Get_Min_Frames_To_Sleep(void)
{
  return (SYS_PWR_MIN_FRAMES_TO_SLEEP + SYS_PWR_AUX_TIMER_MIN_RESOLUTION);

}

